/* eslint-disable no-magic-numbers */
import { useAppSelector } from '@/redux/hooks/useAppSelector';
import { mapEditToolsLayerImageObjectSelector } from '@/redux/mapEditTools/mapEditTools.selectors';
import { useAppDispatch } from '@/redux/hooks/useAppDispatch';
import { MapDrawActionsButton } from '@/components/Map/MapDrawActions/MapDrawActionsButton.component';
import { openLayerImageObjectEditFactoryModal } from '@/redux/modal/modal.slice';
import { updateLayerImageObject } from '@/redux/layers/layers.thunks';
import { mapModelIdSelector } from '@/redux/map/map.selectors';
import { layersActiveLayerSelector } from '@/redux/layers/layers.selectors';
import { layerUpdateImage } from '@/redux/layers/layers.slice';
import { useMapInstance } from '@/utils/context/mapInstanceContext';
import { mapEditToolsSetLayerObject } from '@/redux/mapEditTools/mapEditTools.slice';
import updateGlyph from '@/components/Map/MapViewer/MapViewerVector/utils/shapes/elements/Glyph/updateGlyph';

type MapDrawEditActionsComponentProps = {
  toggleMapEditAction: () => void;
  isActive: boolean;
};

export const MapDrawEditActionsComponent = ({
  toggleMapEditAction,
  isActive,
}: MapDrawEditActionsComponentProps): React.JSX.Element => {
  const currentModelId = useAppSelector(mapModelIdSelector);
  const activeLayer = useAppSelector(layersActiveLayerSelector);
  const layerImageObject = useAppSelector(mapEditToolsLayerImageObjectSelector);
  const dispatch = useAppDispatch();
  const { mapInstance } = useMapInstance();

  const editMapObject = (): void => {
    dispatch(openLayerImageObjectEditFactoryModal());
  };

  const updateZIndex = async (value: number): Promise<void> => {
    if (!activeLayer || !layerImageObject) {
      return;
    }
    const layerImage = await dispatch(
      updateLayerImageObject({
        modelId: currentModelId,
        layerId: activeLayer,
        ...layerImageObject,
        z: layerImageObject.z + value,
      }),
    ).unwrap();
    if (layerImage) {
      dispatch(layerUpdateImage({ modelId: currentModelId, layerId: activeLayer, layerImage }));
      dispatch(mapEditToolsSetLayerObject(layerImage));
      updateGlyph(mapInstance, activeLayer, layerImage);
    }
  };

  return (
    <div className="flex flex-row-reverse gap-4">
      <MapDrawActionsButton
        isActive={isActive}
        toggleMapEditAction={toggleMapEditAction}
        icon="pencil"
        title="Edit image"
      />
      {layerImageObject && (
        <>
          <MapDrawActionsButton
            isActive={false}
            toggleMapEditAction={() => editMapObject()}
            icon="edit-image"
            title="Edit image"
          />
          <MapDrawActionsButton
            isActive={false}
            toggleMapEditAction={() => {}}
            icon="trash"
            title="Remove image"
          />
          <MapDrawActionsButton
            isActive={false}
            toggleMapEditAction={() => updateZIndex(1)}
            icon="arrow-double-up"
            title="Remove image"
          />
          <MapDrawActionsButton
            isActive={false}
            toggleMapEditAction={() => updateZIndex(-1)}
            icon="arrow-double-down"
            title="Remove image"
          />
        </>
      )}
    </div>
  );
};
